/* $NetBSD: t_lwpctl.c,v 1.1 2009/02/20 21:39:57 jmmv Exp $ */

/*-
 * Copyright (c) 2008 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__COPYRIGHT("@(#) Copyright (c) 2008\
 The NetBSD Foundation, inc. All rights reserved.");
__RCSID("$NetBSD: t_lwpctl.c,v 1.1 2009/02/20 21:39:57 jmmv Exp $");

#include <sys/lwpctl.h>

#include <lwp.h>
#include <stdio.h>
#include <time.h>

#include <atf-c.h>

#include "../h_macros.h"

ATF_TC(lwpctl);
ATF_TC_HEAD(lwpctl, tc)
{
	atf_tc_set_md_var(tc, "descr", "Checks lwpctl preemption counter");
}
ATF_TC_BODY(lwpctl, tc)
{
	lwpctl_t *lc;
	struct timespec ts;
	int ctr1, ctr2;

	RL(_lwp_ctl(LWPCTL_FEATURE_PCTR, &lc));

	/* Ensure that preemption is reported. */
	ctr1 = lc->lc_pctr;
	(void)printf("pctr = %d\n", ctr1);
	ts.tv_nsec = 10*1000000;
	ts.tv_sec = 0;
	RL(nanosleep(&ts, 0));
	ctr2 = lc->lc_pctr;
	(void)printf("pctr = %d\n", ctr2);

	ATF_REQUIRE_MSG(ctr1 != ctr2, "counters match");
}

ATF_TP_ADD_TCS(tp)
{
	ATF_TP_ADD_TC(tp, lwpctl);

	return atf_no_error();
}
